<?php
//////////////////////////////////////////////////////////////////////////////////
//    					 copyright (c) Arfooo Annuaire                          //
//   				 by Hocine Guillaume (c) 2007 - 2008                        //
//       					http://www.arfooo.com/                              //
//    Licence Creative Commons http://creativecommons.org/licenses/by/2.0/fr/   //
//////////////////////////////////////////////////////////////////////////////////

/**
 * Abstract class Controller from MVC design pattern
 */
abstract class Controller extends Object
{    
    public $action;
    protected $userId;
    
    /**
     * @var Session session
     */
    public $session;
    
    /**
     * @var Request request
     */
    protected $request;
    
    protected $acl;
    
    protected $autoRender = true;
    public $name;
    public $viewVars = array();
    public $viewFile;
    protected $viewClass = "TemplateLiteView";
    public $localDir     = "";
    public $niceName  = "";
    protected $models = array();
    protected $data;
    
    /**
     * Generates the standard Controller object
     */
    public function __construct()
    {                          
        $this->name = substr(get_class($this), 0, -10);// "Controller" length 
        
        $this->session = Session::getInstance();
        $this->request = Request::getInstance();
        $this->acl = Acl::getInstance();
        $this->userId = $this->session->get("userId");
        
        $names = explode("_", $this->name);
        
        $this->niceName = $names[count($names)-1];
        $this->niceName[0] = strtolower($this->niceName[0]); 
        
        foreach($this->models as $model_name)
        {
            unset($this->$model_name);
        }
    }
    
    public function __get($name)
    {
        $this->$name = Model::factoryInstance($name); 
        return $this->$name;
    }

    /**
     * Set variable which will be passed to View 
     * @param  string|array $p1 containe name or association array
     * @param  string $p2 value of variable
     */   
    protected function set($p1, $p2 = null)
    {
        if($p2 === null && is_array($p1))
        {
            foreach($p1 as $key => $val)
            {
                $this->viewVars[$key] = $val;
            }
        }
        else
        { 
            $this->viewVars[$p1] = $p2;
        }
    }
    
    /**
     * Set error/info messages which will be passed to View 
     * @param  string $msg Message
     */         
    protected function throwInfo($msg)
    {
        if($msg{0} == "-")
          $this->set("errorMessage", substr($msg, 1));  
        else
          $this->set("okMessage", $msg); 
    }

    /**
     * Create module link to navigate between actions in controller 
     * @param  string $action Action name in controller
     * @return string full url to action
     */      
    protected function moduleLink($action = "", $absolute = true)
    {        
        $url = "";//Config::get("SERVER_URL");

        if($this->localDir)
        {
            $url .= "/".rtrim($this->localDir, "/")."";
        }
   
        $url = FrontController::getRewrittedUrl($url."/".$this->niceName."/".$action, $absolute);
        
        if($url[0] != "/" && $absolute)$url = "/".$url;
        
        return $url;
    }

    /**
     * Render associated View 
     */       
    public function render()
    {
        if(!$this->autoRender)return "";

        $view = new $this->viewClass($this);

        return $view->render();
    }

    /**
     * Redirect to spedified url
     * @param  string $url Url    
     */      
    protected function redirect($url, $httpCode = 302)
    {
        header("Location: $url", true, $httpCode);
        exit;
    }
    
    protected function return404()
    {
        header('Status: 404 Not Found');
        header('HTTP/1.0 404 Not Found');
        $this->niceName = "front";
        $this->viewFile = "404";
        Display::set("adPage", "error404");
        echo $this->render();
        exit;
    }

}
?>